package main

import (
	"code.google.com/p/go.net/websocket"
	"flag"
	"log"
	"math/rand"
	"net/http"
	_ "net/http/pprof"
	"os"
	"runtime/pprof"
	"time"
)

var addr = flag.String("addr", ":8666", "http service address")
var tick = flag.Int("tick", 60, "")
var timescale = flag.Float64("timescale", 1, "time scale factor")
var verbose = flag.Bool("verbose", false, "")
var width = flag.Float64("width", 800, "width of field")
var height = flag.Float64("height", 550, "height of field")
var obstacle_count = flag.Int("obstacles", 5, "how many obstacles")
var profile = flag.String("pprof", "", "if specified will run with pprof")
var netprofile = flag.Bool("netprof", false, "if specified will run with net/http/pprof")
var debug = flag.Bool("debug", false, "automatically create games if they don't exist")

var delta float32

var idg *IdGenerator

// This is the main, global collection of games
var games MapLock

func main() {
	log.Printf("Starting Server...")

	rand.Seed(time.Now().UnixNano())
	flag.Parse()
	if *profile != "" {
		f, err := os.Create(*profile)
		if err != nil {
			log.Fatal(err)
		}
		pprof.StartCPUProfile(f)
	}
	if *netprofile {
		go func() {
			log.Println(http.ListenAndServe("localhost:8667", nil))
		}()
	}

	games = MapLock{m: make(map[string]*game)}
	idg = NewIdGenerator()

	delta = (float32(*tick) / 1000.0) * float32(*timescale)

	sm := http.NewServeMux()

	sm.Handle("/", JsonHandler(index))
	sm.Handle("/ws/", websocket.Handler(addPlayer))
	sm.Handle("/game/start/", JsonHandler(startGame))
	sm.Handle("/game/list/", JsonHandler(listGames))
	sm.HandleFunc("/game/stop/", stopGame)
	sm.HandleFunc("/fuck/shit/up/", killServer)

	err := http.ListenAndServe(*addr, sm)
	if err != nil {
		log.Fatal("unable to start server")
	}
}
